@@ -1,7 +1,6 @@
 # -*- coding: utf-8; mode: tcl; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- vim:fenc=utf-8:filetype=tcl:et:sw=4:ts=4:sts=4
-# $Id: portconfigure.tcl 126863 2014-10-16 18:13:52Z jmr@macports.org $
 #
-# Copyright (c) 2007 - 2014 The MacPorts Project
+# Copyright (c) 2007 - 2015 The MacPorts Project
 # Copyright (c) 2007 Markus W. Weissmann <mww@macports.org>
 # Copyright (c) 2002 - 2003 Apple Inc.
 # All rights reserved.
@@ -42,6 +41,67 @@ target_prerun ${org.macports.configure} portconfigure::configure_start
 namespace eval portconfigure {
 }
 
+
+# ********** BEGIN C++ / OBJECTIVE-C++ **********
+
+options configure.cxx \
+        configure.cxx_archflags \
+        configure.cxx_stdlib \
+        configure.cxxflags \
+        configure.objcxx \
+        configure.objcxx_archflags \
+        configure.objcxxflags \
+        configure.universal_cxxflags \
+        configure.universal_objcxxflags
+
+default configure.cxx                   {[portconfigure::configure_get_compiler cxx]}
+default configure.cxx_archflags         {[portconfigure::configure_get_archflags cxx]}
+default configure.cxx_stdlib            {$cxx_stdlib}
+default configure.cxxflags \
+        {[portconfigure::construct_cxxflags ${configure.optflags}]}
+default configure.objcxx                {[portconfigure::configure_get_compiler objcxx]}
+default configure.objcxx_archflags      {[portconfigure::configure_get_archflags objcxx]}
+# No current reason for OBJCXXFLAGS to differ from CXXFLAGS.
+default configure.objcxxflags           {${configure.cxxflags}}
+default configure.universal_cxxflags    {[portconfigure::configure_get_universal_cflags]}
+default configure.universal_objcxxflags {${configure.universal_cxxflags}}
+
+# Don't let Portfiles trash "-stdlib"; if they want to remove it, they
+# should clear configure.cxx_stdlib.
+option_proc configure.cxxflags portconfigure::stdlib_trace
+option_proc configure.objcxxflags portconfigure::stdlib_trace
+
+proc portconfigure::should_add_stdlib {} {
+    set has_stdlib [expr {[option configure.cxx_stdlib] ne ""}]
+    set is_clang [string match *clang* [option configure.cxx]]
+    return [expr {$has_stdlib && $is_clang}]
+}
+proc portconfigure::should_add_cxx_abi {} {
+    set is_oldos [expr {[option os.platform] eq "darwin" && [option os.major] < 10}]
+    set is_mp_gcc [string match *g++-mp-* [option configure.cxx]]
+    return [expr {$is_oldos && $is_mp_gcc}]
+}
+proc portconfigure::construct_cxxflags {flags} {
+    if {[portconfigure::should_add_stdlib]} {
+        lappend flags -stdlib=[option configure.cxx_stdlib]
+    } elseif {[portconfigure::should_add_cxx_abi]} {
+        lappend flags -D_GLIBCXX_USE_CXX11_ABI=0
+    }
+    return $flags
+}
+proc portconfigure::stdlib_trace {opt action args} {
+    foreach flag [lsearch -all -inline [option $opt] -stdlib=*] {
+        $opt-delete $flag
+    }
+    if {$action eq "read" && [portconfigure::should_add_stdlib]} {
+        $opt-append -stdlib=[option configure.cxx_stdlib]
+    }
+    return
+}
+
+# ********** END C++ / OBJECTIVE-C++ **********
+
+
 # define options
 commands configure autoreconf automake autoconf xmkmf
 # defaults
@@ -79,7 +139,9 @@ proc portconfigure::add_build_dep { type dep } {
         ([info exists option_defaults(${type}.cmd)] && [set ${type}.cmd] eq $option_defaults(${type}.cmd)) ||
         (![info exists option_defaults(${type}.cmd)] && [set ${type}.cmd] eq ${type})
         )} {
-            eval [linsert $dep 0 depends_build-append]
+            # Add dependencies if they are not already in the list
+            depends_build-delete {*}$dep
+            depends_build-append {*}$dep
     }
 }
 
@@ -99,14 +161,14 @@ proc portconfigure::set_configure_type {option action args} {
             autoreconf.cmd  -
             automake.cmd    -
             autoconf.cmd {
-                eval [linsert $configure_map(autoconf) 0 depends_build-delete]
+                depends_build-delete {*}$configure_map(autoconf)
             }
             xmkmf.cmd {
-                eval [linsert $configure_map(xmkmf) 0 depends_build-delete]
+                depends_build-delete {*}$configure_map(xmkmf)
             }
             use_xmkmf {
                 if {[tbool args]} {
-                    eval [linsert $configure_map(xmkmf) 0 depends_build-append]
+                    depends_build-append {*}$configure_map(xmkmf)
                 }
             }
             default {
@@ -130,17 +192,15 @@ default configure.march     {}
 default configure.mtune     {}
 # We could have debug/optimizations be global configurable at some point.
 options configure.optflags \
-        configure.cflags configure.cxxflags \
-        configure.objcflags configure.objcxxflags \
+        configure.cflags \
+        configure.objcflags \
         configure.cppflags configure.ldflags configure.libs \
         configure.fflags configure.f90flags configure.fcflags \
         configure.classpath
 # compiler flags section
 default configure.optflags      {-Os}
 default configure.cflags        {${configure.optflags}}
-default configure.cxxflags      {${configure.optflags}}
 default configure.objcflags     {${configure.optflags}}
-default configure.objcxxflags   {${configure.optflags}}
 default configure.cppflags      {-I${prefix}/include}
 default configure.ldflags       {"-L${prefix}/lib -Wl,-headerpad_max_install_names"}
 default configure.libs          {}
@@ -168,33 +228,31 @@ default configure.build_arch    {[portconfigure::choose_supported_archs ${build_
 default configure.ld_archflags  {[portconfigure::configure_get_ld_archflags]}
 default configure.sdk_version   {$macosx_sdk_version}
 default configure.sdkroot       {[portconfigure::configure_get_sdkroot ${configure.sdk_version}]}
-foreach tool {cc cxx objc objcxx f77 f90 fc} {
+foreach tool {cc objc f77 f90 fc} {
     options configure.${tool}_archflags
     default configure.${tool}_archflags  "\[portconfigure::configure_get_archflags $tool\]"
 }
 
 options configure.universal_archs configure.universal_args \
-        configure.universal_cflags configure.universal_cxxflags \
-        configure.universal_objcflags configure.universal_objcxxflags \
+        configure.universal_cflags \
+        configure.universal_objcflags \
         configure.universal_cppflags configure.universal_ldflags
 default configure.universal_archs       {[portconfigure::choose_supported_archs ${universal_archs}]}
 default configure.universal_args        {--disable-dependency-tracking}
 default configure.universal_cflags      {[portconfigure::configure_get_universal_cflags]}
-default configure.universal_cxxflags    {[portconfigure::configure_get_universal_cflags]}
 default configure.universal_objcflags   {${configure.universal_cflags}}
-default configure.universal_objcxxflags {${configure.universal_cxxflags}}
 default configure.universal_cppflags    {}
 default configure.universal_ldflags     {[portconfigure::configure_get_universal_ldflags]}
 
 # Select a distinct compiler (C, C preprocessor, C++)
 options configure.ccache configure.distcc configure.pipe configure.cc \
-        configure.cxx configure.cpp configure.objc configure.objcxx configure.f77 \
+        configure.cpp configure.objc configure.f77 \
         configure.f90 configure.fc configure.javac configure.compiler \
         compiler.blacklist compiler.whitelist compiler.fallback
 default configure.ccache        {${configureccache}}
 default configure.distcc        {${configuredistcc}}
 default configure.pipe          {${configurepipe}}
-foreach tool {cc cxx objc objcxx cpp f77 f90 fc javac} {
+foreach tool {cc objc cpp f77 f90 fc javac} {
     default configure.$tool     "\[portconfigure::configure_get_compiler $tool\]"
 }
 default configure.compiler      {[portconfigure::configure_get_default_compiler]}
@@ -202,10 +260,6 @@ default compiler.fallback       {[portconfigure::get_compiler_fallback]}
 default compiler.blacklist      {}
 default compiler.whitelist      {}
 
-# Select a C++ STL implementation
-options configure.cxx_stdlib
-default configure.cxx_stdlib    {$cxx_stdlib}
-
 set_ui_prefix
 
 proc portconfigure::configure_start {args} {
@@ -221,7 +275,7 @@ proc portconfigure::configure_start {args} {
         {^gcc$}                             {System GCC}
         {^gcc-(3\.3|4\.[02])$}              {Xcode GCC %s}
         {^llvm-gcc-4\.2$}                   {Xcode LLVM-GCC 4.2}
-        {^macports-clang$}                  {MacPorts Clang (port select}
+        {^macports-clang$}                  {MacPorts Clang (port select)}
         {^macports-clang-(\d+\.\d+)$}       {MacPorts Clang %s}
         {^macports-dragonegg-(\d+\.\d+)$}   {MacPorts DragonEgg %s}
         {^macports-dragonegg-(\d+\.\d+)-gcc-(\d+\.\d+)$}
@@ -232,13 +286,14 @@ proc portconfigure::configure_start {args} {
     }
     foreach {re fmt} $valid_compilers {
         if {[set matches [regexp -inline $re $compiler]] ne ""} {
-            set compiler_name [eval [linsert [lrange $matches 1 end] 0 format $fmt]]
+            set compiler_name [format $fmt {*}[lrange $matches 1 end]]
             break
         }
     }
     if {![info exists compiler_name]} {
         return -code error "Invalid value for configure.compiler: $compiler"
     }
+    ui_debug "Preferred compilers: [option compiler.fallback]"
     ui_debug "Using compiler '$compiler_name'"
 
     # Additional ccache directory setup
@@ -277,16 +332,16 @@ proc portconfigure::choose_supported_archs {archs} {
     }
     set ret {}
     foreach arch $archs {
-        if {[lsearch -exact $supported_archs $arch] != -1} {
+        if {$arch in $supported_archs} {
             set add_arch $arch
-        } elseif {$arch eq "x86_64" && [lsearch -exact $supported_archs "i386"] != -1} {
+        } elseif {$arch eq "x86_64" && "i386" in $supported_archs} {
             set add_arch "i386"
-        } elseif {$arch eq "ppc64" && [lsearch -exact $supported_archs "ppc"] != -1} {
+        } elseif {$arch eq "ppc64" && "ppc" in $supported_archs} {
             set add_arch "ppc"
         } else {
             continue
         }
-        if {[lsearch -exact $ret $add_arch] == -1} {
+        if {$add_arch ni $ret} {
             lappend ret $add_arch
         }
     }
@@ -319,7 +374,7 @@ proc portconfigure::configure_get_archflags {tool} {
 # Unfortunately there's no consistent way to do this when the compiler
 # doesn't support -arch, because it could be used to link rather than using
 # ld directly. So we punt and let portfiles deal with that case.
-proc portconfigure::configure_get_ld_archflags {args} {
+proc portconfigure::configure_get_ld_archflags {} {
     global configure.build_arch configure.compiler
     if {${configure.build_arch} ne "" && [arch_flag_supported ${configure.compiler}]} {
         return "-arch ${configure.build_arch}"
@@ -350,7 +405,7 @@ proc portconfigure::configure_get_sdkroot {sdk_version} {
 }
 
 # internal function to determine the "-arch xy" flags for the compiler
-proc portconfigure::configure_get_universal_archflags {args} {
+proc portconfigure::configure_get_universal_archflags {} {
     global configure.universal_archs
     set flags ""
     foreach arch ${configure.universal_archs} {
@@ -364,12 +419,12 @@ proc portconfigure::configure_get_universal_archflags {args} {
 }
 
 # internal function to determine the CFLAGS for the compiler
-proc portconfigure::configure_get_universal_cflags {args} {
+proc portconfigure::configure_get_universal_cflags {} {
     return [configure_get_universal_archflags]
 }
 
 # internal function to determine the LDFLAGS for the compiler
-proc portconfigure::configure_get_universal_ldflags {args} {
+proc portconfigure::configure_get_universal_ldflags {} {
     return [configure_get_universal_archflags]
 }
 
@@ -387,7 +442,7 @@ proc portconfigure::compiler_port_name {compiler} {
     }
     foreach {re fmt} $valid_compiler_ports {
         if {[set matches [regexp -inline $re $compiler]] ne ""} {
-            return [eval [linsert [lrange $matches 1 end] 0 format $fmt]]
+            return [format $fmt {*}[lrange $matches 1 end]]
         }
     }
     return {}
@@ -398,7 +453,7 @@ proc portconfigure::compiler_is_port {compiler} {
 }
 
 # internal function to determine the default compiler
-proc portconfigure::configure_get_default_compiler {args} {
+proc portconfigure::configure_get_default_compiler {} {
     if {[option compiler.whitelist] ne ""} {
         set search_list [option compiler.whitelist]
     } else {
@@ -425,30 +480,83 @@ proc portconfigure::configure_get_default_compiler {args} {
 
 # internal function to choose compiler fallback list based on platform
 proc portconfigure::get_compiler_fallback {} {
-    global xcodeversion macosx_deployment_target default_compilers configure.sdkroot
+    global xcodeversion macosx_deployment_target default_compilers configure.sdkroot configure.cxx_stdlib os.major
+
+    # Check our override
     if {[info exists default_compilers]} {
         return $default_compilers
-    } elseif {$xcodeversion eq "none" || $xcodeversion eq ""} {
-        return {cc}
-    } elseif {[vercmp $xcodeversion 6.0] >= 0 && [vercmp $macosx_deployment_target 10.10] >= 0} {
-        return {clang macports-clang-3.4 macports-clang-3.3}
-    } elseif {[vercmp $xcodeversion 5.0] >= 0} {
-        return {clang macports-clang-3.4 macports-clang-3.3 macports-llvm-gcc-4.2 apple-gcc-4.2}
-    } elseif {[vercmp $xcodeversion 4.3] >= 0} {
-        return {clang llvm-gcc-4.2 macports-clang-3.4 macports-clang-3.3 apple-gcc-4.2}
-    } elseif {[vercmp $xcodeversion 4.0] >= 0} {
-        return {llvm-gcc-4.2 clang gcc-4.2 macports-clang-3.4 macports-clang-3.3 apple-gcc-4.2}
-    } elseif {[vercmp $xcodeversion 3.2] >= 0} {
-        if {[string match *10.4u* ${configure.sdkroot}]} {
-            return {gcc-4.0}
-        } else {
-            return {gcc-4.2 clang llvm-gcc-4.2 macports-clang-3.4 macports-clang-3.3 macports-llvm-gcc-4.2 apple-gcc-4.2 gcc-4.0}
-        }
-    } elseif {[vercmp $xcodeversion 3.0] >= 0} {
-        return {gcc-4.2 apple-gcc-4.2 gcc-4.0 macports-clang-3.4 macports-clang-3.3}
-    } else {
-        return {apple-gcc-4.2 gcc-4.0 gcc-3.3 macports-clang-3.3}
     }
+
+    # Check for platforms without Xcode
+    if {$xcodeversion eq "none" || $xcodeversion eq ""} {
+        return {cc}
+    }
+
+    # Legacy cases
+    if {[vercmp $xcodeversion 4.0] < 0} {
+        set canonical_archs [get_canonical_archs]
+        if {[vercmp $xcodeversion 3.2] >= 0} {
+            if {[string match *10.4u* ${configure.sdkroot}]} {
+                return {gcc-4.0}
+            }
+        } elseif {[vercmp $xcodeversion 3.0] >= 0} {
+            if {"ppc" in $canonical_archs || "ppc64" in $canonical_archs} {
+                return {gcc-4.2 apple-gcc-4.2 gcc-4.0 macports-gcc-6 macports-gcc-7}
+            } else {
+                return {gcc-4.2 apple-gcc-4.2 gcc-4.0 macports-clang-3.4 macports-clang-3.3}
+            }
+        } else {
+            # Xcode 2.x (Tiger)
+            if {"ppc" in $canonical_archs || "ppc64" in $canonical_archs} {
+                if {"i386" in $canonical_archs} {
+                    # universal
+                    return {apple-gcc-4.2 gcc-4.0 macports-gcc-6 macports-gcc-7}
+                } else {
+                    # ppc only
+                    return {apple-gcc-4.2 gcc-4.0 gcc-3.3 macports-gcc-6 macports-gcc-7}
+                }
+            } else {
+                # i386 only
+                return {apple-gcc-4.2 gcc-4.0 macports-clang-3.3}
+            }
+        }
+    }
+
+    set compilers {}
+
+    # Set our preferred Xcode-provided compilers
+    if {[vercmp $xcodeversion 5.0] >= 0} {
+        lappend compilers clang
+    } elseif {[vercmp $xcodeversion 4.3] >= 0} {
+        lappend compilers clang llvm-gcc-4.2
+    } elseif {[vercmp $xcodeversion 4.0] >= 0} {
+        lappend compilers llvm-gcc-4.2 clang
+    } else {
+        lappend compilers gcc-4.2 clang llvm-gcc-4.2
+    }
+
+    # Determine which versions of clang we prefer
+    if {${configure.cxx_stdlib} eq "libc++"} {
+        # clang-3.5+ require libc++
+        lappend compilers macports-clang-4.0
+
+        if {${os.major} < 17} {
+            # The High Sierra SDK requires a toolchain that can apply nullability to uuid_t
+            lappend compilers macports-clang-3.9
+        }
+
+        if {${os.major} < 16} {
+            # The Sierra SDK requires a toolchain that supports class properties
+            lappend compilers macports-clang-3.7
+        }
+    }
+
+    if {${os.major} < 16} {
+        # We dropped support for these compilers on Sierra
+        lappend compilers macports-clang-3.4 macports-llvm-gcc-4.2 apple-gcc-4.2
+    }
+
+    return $compilers
 }
 
 # Find a developer tool
@@ -702,7 +810,7 @@ proc portconfigure::configure_main {args} {
             PERL PYTHON RUBY INSTALL AWK BISON PKG_CONFIG PKG_CONFIG_PATH \
         } {
             set value [option configure.[string tolower $env_var]]
-            eval [linsert $value 0 append_to_environment_value configure $env_var]
+            append_to_environment_value configure $env_var {*}$value
         }
 
         # https://trac.macports.org/ticket/34221
@@ -713,22 +821,22 @@ proc portconfigure::configure_main {args} {
         # add SDK flags if cross-compiling (or universal on ppc tiger)
         if {${configure.sdkroot} ne ""} {
             foreach env_var {CPPFLAGS CFLAGS CXXFLAGS OBJCFLAGS OBJCXXFLAGS} {
-                append_to_environment_value configure $env_var -isysroot ${configure.sdkroot}
+                append_to_environment_value configure $env_var -isysroot${configure.sdkroot}
             }
             append_to_environment_value configure "LDFLAGS" -Wl,-syslibroot,${configure.sdkroot}
         }
 
         # add extra flags that are conditional on whether we're building universal
-        eval [linsert [get_canonical_archflags cc] 0 append_to_environment_value configure CFLAGS]
+        append_to_environment_value configure CFLAGS {*}[get_canonical_archflags cc]
         foreach tool {cxx objc objcxx cpp f77 f90 fc ld} {
             if {[catch {get_canonical_archflags $tool} flags]} {
                 continue
             }
             set env_var [string toupper $tool]FLAGS
-            eval [linsert $flags 0 append_to_environment_value configure $env_var]
+            append_to_environment_value configure $env_var {*}$flags
         }
         if {[variant_exists universal] && [variant_isset universal]} {
-            eval [linsert ${configure.universal_args} 0 configure.pre_args-append]
+            configure.pre_args-append {*}${configure.universal_args}
         } else {
             foreach env_var {CFLAGS CXXFLAGS OBJCFLAGS OBJCXXFLAGS FFLAGS F90FLAGS FCFLAGS LDFLAGS} {
                 if {${configure.march} ne ""} {
@@ -740,12 +848,6 @@ proc portconfigure::configure_main {args} {
             }
         }
 
-        # Add flags to specify C++ STL implementation
-        if {${configure.cxx_stdlib} ne "" && [string match "*clang*" [option configure.cxx]]} {
-            append_to_environment_value configure CXXFLAGS -stdlib=${configure.cxx_stdlib}
-            append_to_environment_value configure OBJCXXFLAGS -stdlib=${configure.cxx_stdlib}
-        }
-
         # Execute the command (with the new environment).
         if {[catch {command_exec configure} result]} {
             global configure.dir
